package be.iminds.ilabt.jfed.testing.shared;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

/**
 * Proxy configuration
 */
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value = {"@context"})
public class Proxy {
    public enum ProxyMode { OFF, MANUAL, USER_AUTHORITY };
    public enum ProxyKeySource { USER, NODE };

    @Nonnull
    private final ProxyMode mode; //valid option: off (default), manual(use preferences below), user(automatically use proxy of user authority)

    @Nullable
    private final String hostName;
    @Nullable
    private final String hostKey;
    private final int hostPort;
    @Nonnull
    private final String username; //default: same username as logged in user (same as "$USER"). Some limited variable substitution is done:
    // - $NODE: username for target node
    // - $USER: logged in user name (last part of user URN)
    // - $SLICE: slice name (last part of slice URN)
    @Nonnull
    private final ProxyKeySource keySource; //valid choices:
    // - user (take SSH keys from user login cert and key)
    // - node (use the same SSH keys as the one to login to the actual target node, these are typically generated by NodeLoginTestStep, unless fixed_ssh_public_key_file says otherwise)


    public Proxy() {
        this.mode = ProxyMode.OFF;
        this.hostName = null;
        this.hostKey = null;
        this.hostPort = 22;
        this.username = "$USER";
        this.keySource = ProxyKeySource.USER;
    }

    @JsonCreator
    public Proxy(@Nullable @JsonProperty("mode") ProxyMode mode,
                 @Nullable @JsonProperty("hostName") String hostName,
                 @Nullable @JsonProperty("hostKey") String hostKey,
                 @Nullable @JsonProperty("hostPort") Integer hostPort,
                 @Nullable @JsonProperty("username") String username,
                 @Nullable @JsonProperty("keySource") ProxyKeySource keySource) {
        this.mode = mode == null ? new Proxy().getMode() : mode;
        this.hostName = hostName;
        this.hostKey = hostKey;
        this.hostPort = hostPort == null ? new Proxy().getHostPort() : hostPort;
        this.username = username == null ? new Proxy().getUsername() : username;
        this.keySource = keySource == null ? new Proxy().getKeySource() : keySource;
    }

    @JsonIgnore
    public boolean isValid() {
        if (mode == ProxyMode.OFF)
            return true;
        if (mode == ProxyMode.USER_AUTHORITY)
            return true;

        if (getHostName() == null)
            throw new IllegalArgumentException("Config Error: You must specify proxy.hostName if proxy.mode == MANUAL");
        if (getHostKey() == null)
            throw new IllegalArgumentException("Config Error: You must specify proxy.hostKey if proxy.mode == MANUAL");

        return true;
    }

    @JsonProperty
    @Nonnull
    public ProxyMode getMode() {
        return mode;
    }

    @JsonProperty
    @Nullable
    public String getHostName() {
        return hostName;
    }

    @JsonProperty
    @Nullable
    public String getHostKey() {
        return hostKey;
    }

    @JsonProperty
    public int getHostPort() {
        return hostPort;
    }

    @JsonProperty
    @Nonnull
    public String getUsername() {
        return username;
    }

    @JsonProperty
    @Nonnull
    public ProxyKeySource getKeySource() {
        return keySource;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Proxy)) return false;

        Proxy proxy = (Proxy) o;

        if (hostPort != proxy.hostPort) return false;
        if (mode != proxy.mode) return false;
        if (hostName != null ? !hostName.equals(proxy.hostName) : proxy.hostName != null) return false;
        if (hostKey != null ? !hostKey.equals(proxy.hostKey) : proxy.hostKey != null) return false;
        if (!username.equals(proxy.username)) return false;
        return keySource == proxy.keySource;
    }

    @Override
    public int hashCode() {
        int result = mode.hashCode();
        result = 31 * result + (hostName != null ? hostName.hashCode() : 0);
        result = 31 * result + (hostKey != null ? hostKey.hashCode() : 0);
        result = 31 * result + hostPort;
        result = 31 * result + username.hashCode();
        result = 31 * result + keySource.hashCode();
        return result;
    }
}
